import org.junit.Test;

import javax.tools.JavaCompiler;
import javax.tools.ToolProvider;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static org.junit.Assert.*;

public class ClassSampleTest {
    private File root = new File(new File("").getAbsolutePath());

    public void deleteFile(String name) {
        new File(root, name + ".java").delete();
        new File(root, name + ".class").delete();
    }

    @Test
    public void testOop1() throws InterruptedException, IOException, ClassNotFoundException, NoSuchFieldException {
        OopType type = OopType.ENUM;
        String name = "Month";
        Map<String, String> properties = Map.of("", "NAME");
        String code = ClassUtil.classMaker(
            new ClassDetail(type, name, properties, List.of(), List.of())
        );

        Thread.sleep(100);

        File sourceFile = new File(root, name + ".java");
        Files.write(sourceFile.toPath(), code.getBytes(StandardCharsets.UTF_8));
        JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();
        compiler.run(null, null, null, sourceFile.getPath());
        URLClassLoader classLoader = URLClassLoader.newInstance(new URL[] { root.toURI().toURL() });
        Class<?> cls = Class.forName(name, true, classLoader);
        assertEquals("NAME", cls.getDeclaredField("NAME").getName());

        deleteFile(name);
    }

    @Test
    public void testOop2() throws InterruptedException, IOException, ClassNotFoundException, NoSuchFieldException {
        OopType type = OopType.CLASS;
        String name = "Month";
        Map<String, String> properties = Map.of("String", "name");
        List<String> oop = List.of("Object");
        String code = ClassUtil.classMaker(new ClassDetail(type, name, properties, List.of(), oop));

        Thread.sleep(100);

        File sourceFile = new File(root, name + ".java");
        Files.write(sourceFile.toPath(), code.getBytes(StandardCharsets.UTF_8));
        JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();
        compiler.run(null, null, null, sourceFile.getPath());
        URLClassLoader classLoader = URLClassLoader.newInstance(new URL[] { root.toURI().toURL() });
        Class<?> cls = Class.forName(name, true, classLoader);
        assertEquals("name", cls.getDeclaredField("name").getName());

        deleteFile(name);
    }
}
